"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeMirror = exports.makeLineNumbers = void 0;
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var autocomplete_1 = require("@codemirror/autocomplete");
var commands_1 = require("@codemirror/commands");
var lang_cpp_1 = require("@codemirror/lang-cpp");
var lang_markdown_1 = require("@codemirror/lang-markdown");
var lang_javascript_1 = require("@codemirror/lang-javascript");
var lang_html_1 = require("@codemirror/lang-html");
var lang_css_1 = require("@codemirror/lang-css");
var lang_xml_1 = require("@codemirror/lang-xml");
var lang_java_1 = require("@codemirror/lang-java");
var lang_json_1 = require("@codemirror/lang-json");
var lang_less_1 = require("@codemirror/lang-less");
var lang_php_1 = require("@codemirror/lang-php");
var lang_python_1 = require("@codemirror/lang-python");
var lang_sql_1 = require("@codemirror/lang-sql");
var lang_rust_1 = require("@codemirror/lang-rust");
var lang_wast_1 = require("@codemirror/lang-wast");
var lang_sass_1 = require("@codemirror/lang-sass");
var language_1 = require("@codemirror/language");
var lint_1 = require("@codemirror/lint");
var search_1 = require("@codemirror/search");
var state_1 = require("@codemirror/state");
var view_1 = require("@codemirror/view");
var style_module_css_1 = tslib_1.__importDefault(require("./style.module.css"));
var languageModes = {
    h: (0, lang_cpp_1.cpp)(),
    c: (0, lang_cpp_1.cpp)(),
    clj: (0, lang_cpp_1.cpp)(),
    coffee: (0, lang_cpp_1.cpp)(),
    cpp: (0, lang_cpp_1.cpp)(),
    cs: (0, lang_cpp_1.cpp)(),
    css: (0, lang_css_1.css)(),
    groovy: (0, lang_cpp_1.cpp)(),
    haxe: (0, lang_cpp_1.cpp)(),
    htm: (0, lang_html_1.html)(),
    html: (0, lang_html_1.html)(),
    java: (0, lang_java_1.java)(),
    js: (0, lang_javascript_1.javascript)(),
    ts: (0, lang_javascript_1.javascript)(),
    jsm: (0, lang_javascript_1.javascript)(),
    json: (0, lang_json_1.json)(),
    less: (0, lang_less_1.less)(),
    lua: (0, lang_cpp_1.cpp)(),
    markdown: (0, lang_markdown_1.markdown)(),
    md: (0, lang_markdown_1.markdown)(),
    mdown: (0, lang_markdown_1.markdown)(),
    mdwn: (0, lang_markdown_1.markdown)(),
    mkd: (0, lang_markdown_1.markdown)(),
    ml: (0, lang_cpp_1.cpp)(),
    mli: (0, lang_cpp_1.cpp)(),
    pl: (0, lang_cpp_1.cpp)(),
    php: (0, lang_php_1.php)(),
    py: (0, lang_python_1.python)(),
    rb: (0, lang_cpp_1.cpp)(),
    rs: (0, lang_rust_1.rust)(),
    sh: (0, lang_cpp_1.cpp)(),
    sql: (0, lang_sql_1.sql)(),
    svg: (0, lang_xml_1.xml)(),
    sass: (0, lang_sass_1.sass)(),
    xml: (0, lang_xml_1.xml)(),
    wast: (0, lang_wast_1.wast)()
    //filetype.txt = 'txt';
};
function detectMode(ext) {
    var mode = languageModes[ext];
    if (mode) {
        return mode;
    }
    return undefined;
    // return autoFold(autoMode());
}
// https://codemirror.net/examples/config/#dynamic-configuration
var lineNumbersCompartment = new state_1.Compartment();
function makeLineNumbers(showLineNumbers) {
    var v = showLineNumbers ? (0, view_1.lineNumbers)() : [];
    return lineNumbersCompartment.of(v);
}
exports.makeLineNumbers = makeLineNumbers;
var basicSetup = [
    (0, view_1.highlightActiveLineGutter)(),
    (0, view_1.highlightSpecialChars)(),
    (0, commands_1.history)(),
    (0, language_1.foldGutter)(),
    (0, view_1.drawSelection)(),
    (0, view_1.dropCursor)(),
    state_1.EditorState.allowMultipleSelections.of(true),
    (0, language_1.indentOnInput)(),
    (0, language_1.syntaxHighlighting)(language_1.defaultHighlightStyle, { fallback: true }),
    (0, language_1.bracketMatching)(),
    (0, autocomplete_1.closeBrackets)(),
    (0, autocomplete_1.autocompletion)(),
    (0, view_1.rectangularSelection)(),
    (0, view_1.crosshairCursor)(),
    (0, view_1.highlightActiveLine)(),
    (0, search_1.highlightSelectionMatches)(),
    view_1.keymap.of(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray(tslib_1.__spreadArray([], autocomplete_1.closeBracketsKeymap, true), commands_1.defaultKeymap, true), search_1.searchKeymap, true), commands_1.historyKeymap, true), language_1.foldKeymap, true), autocomplete_1.completionKeymap, true), lint_1.lintKeymap, true))
];
var createCodeMirrorState = function (_a) {
    var onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable, value = _a.value, fileExtension = _a.fileExtension;
    var mode = detectMode(fileExtension);
    return state_1.EditorState.create({
        doc: value,
        extensions: tslib_1.__spreadArray(tslib_1.__spreadArray([
            basicSetup,
            //https://github.com/kjk/onlinetool.io/blob/752dc581d6e7f38ddd3b28be1feed65e9de860b1/web/CodeMirrorConfig.js#L12
            makeLineNumbers(true)
        ], (mode !== undefined ? [mode] : []), true), [
            view_1.EditorView.updateListener.of(function (v) {
                if (v.focusChanged) {
                    lock.current = v.view.hasFocus;
                }
                if (v.docChanged) {
                    var getString = function () { return v.state.doc.toString(); };
                    onChange(getString);
                }
            }),
            view_1.EditorView.theme({
                '&.cm-focused': {
                    outline: 'none'
                }
            }, { dark: dark }),
            // https://github.com/codemirror/codemirror.next/issues/173
            view_1.EditorView.editable.of(editable)
        ], false)
    });
};
var createCodeMirrorView = function (_a) {
    var root = _a.root, options = tslib_1.__rest(_a, ["root"]);
    return new view_1.EditorView({
        state: createCodeMirrorState(options),
        parent: root
    });
};
exports.CodeMirror = react_1.default.forwardRef(function (_a, ref) {
    var value = _a.value, onChange = _a.onChange, lock = _a.lock, dark = _a.dark, editable = _a.editable, fileExtension = _a.fileExtension;
    var divRef = react_1.default.useRef(null);
    var showLNumbers = react_1.default.useRef(true);
    var editorRef = react_1.default.useRef();
    var _b = react_1.default.useState(false), focus = _b[0], setFocus = _b[1];
    react_1.default.useEffect(function () {
        if (!divRef.current)
            return;
        var editor = createCodeMirrorView({
            root: divRef.current,
            onChange: onChange,
            lock: lock,
            dark: dark,
            editable: editable,
            value: value,
            fileExtension: fileExtension
        });
        editorRef.current = editor;
        /*const timer = setInterval(() => {
          editor.focus();
          if(editor.hasFocus) clearInterval(timer);
        }, 500);*/
        return function () {
            // clearInterval(timer);
            editor.destroy();
        };
    }, [onChange, value, lock, dark]);
    react_1.default.useImperativeHandle(ref, function () { return ({
        update: function (value) {
            var current = editorRef.current;
            if (!current)
                return;
            current.setState(createCodeMirrorState({
                onChange: onChange,
                lock: lock,
                dark: dark,
                editable: editable,
                value: value
            }));
        },
        toggleLineNumbers: function () {
            var current = editorRef.current;
            if (!current)
                return;
            showLNumbers.current = !showLNumbers.current;
            var v = showLNumbers.current ? (0, view_1.lineNumbers)() : [];
            current.dispatch({
                effects: lineNumbersCompartment.reconfigure(v)
            });
        }
    }); });
    return (react_1.default.createElement("div", { onFocus: function () { return setFocus(true); }, onBlur: function () { return setFocus(false); }, className: style_module_css_1.default['code'] + (focus ? ' ' + style_module_css_1.default['focus'] : '') },
        react_1.default.createElement("div", { ref: divRef })));
});
//# sourceMappingURL=CodeMirror.js.map